@@ -68,7 +68,7 @@ long long atoll_b (const char *nptr);
 /* Prototypes for worker functions.  */
 int hogcpu (void);
 int hogio (void);
-int hogvm (long long bytes, long long stride, long long hang);
+int hogvm (long long bytes, long long stride, long long hang, int keep);
 int hoghdd (long long bytes, int clean);
 
 int
@@ -87,6 +87,7 @@ main (int argc, char **argv)
   long long do_vm_bytes = 256 * 1024 * 1024;
   long long do_vm_stride = 4096;
   long long do_vm_hang = -1;
+  int do_vm_keep = 0;
   long long do_hdd = 0;
   int do_hdd_clean = 2;
   long long do_hdd_bytes = 1024 * 1024 * 1024;
@@ -211,6 +212,10 @@ main (int argc, char **argv)
               exit (1);
             }
         }
+      else if (strcmp (arg, "--vm-keep") == 0)
+        {
+          do_vm_keep = 1;
+        }
       else if (strcmp (arg, "--hdd") == 0 || strcmp (arg, "-d") == 0)
         {
           assert_arg ("--hdd");
@@ -336,7 +341,8 @@ main (int argc, char **argv)
               usleep (backoff);
               if (do_dryrun)
                 exit (0);
-              exit (hogvm (do_vm_bytes, do_vm_stride, do_vm_hang));
+              exit (hogvm
+                    (do_vm_bytes, do_vm_stride, do_vm_hang, do_vm_keep));
             case -1:           /* error */
               err (stderr, "fork failed: %s\n", strerror (errno));
               break;
@@ -472,26 +478,30 @@ hogio ()
 }
 
 int
-hogvm (long long bytes, long long stride, long long hang)
+hogvm (long long bytes, long long stride, long long hang, int keep)
 {
   long long i;
   char *ptr;
+  int do_malloc = 1;
 
   while (1)
     {
-      dbg (stdout, "allocating %lli bytes ...\n", bytes);
-      dbg (stdout, "striding by %lli bytes ...\n", stride);
-      if ((ptr = (char *) malloc (bytes * sizeof (char))))
+      if (do_malloc)
         {
-          for (i = 0; i < bytes; i += stride)
-            ptr[i] = 'Z';       /* Ensure that COW happens.  */
-        }
-      else
-        {
-          err (stderr, "hogvm malloc failed: %s\n", strerror (errno));
-          return 1;
+          dbg (stdout, "allocating %lli bytes ...\n", bytes);
+          if (!(ptr = (char *) malloc (bytes * sizeof (char))))
+            {
+              err (stderr, "hogvm malloc failed: %s\n", strerror (errno));
+              return 1;
+            }
+          if (keep)
+            do_malloc = 0;
         }
 
+      dbg (stdout, "touching bytes in strides of %lli bytes ...\n", stride);
+      for (i = 0; i < bytes; i += stride)
+        ptr[i] = 'Z';           /* Ensure that COW happens.  */
+
       if (hang == 0)
         {
           dbg (stdout, "sleeping forever with allocated memory\n");
@@ -504,8 +514,11 @@ hogvm (long long bytes, long long stride, long long hang)
           sleep (hang);
         }
 
-      free (ptr);
-      dbg (stdout, "freed %lli bytes\n", bytes);
+      if (do_malloc)
+        {
+          free (ptr);
+          dbg (stdout, "freed %lli bytes\n", bytes);
+        }
     }
 
   return 0;
@@ -685,7 +698,7 @@ atoll_s (const char *nptr)
       break;
     case 'y':
     case 'Y':
-      factor = 60 * 60 * 24 * 360;
+      factor = 60 * 60 * 24 * 365;
       break;
     default:
       if (suffix < '0' || suffix > '9')
@@ -738,6 +751,7 @@ usage (int status)
     "     --vm-bytes B   malloc B bytes per vm worker (default is 256MB)\n"
     "     --vm-stride B  touch a byte every B bytes (default is 4096)\n"
     "     --vm-hang N    sleep N secs before free (default is none, 0 is inf)\n"
+    "     --vm-keep      redirty memory instead of freeing and reallocating\n"
     " -d, --hdd N        spawn N workers spinning on write()/unlink()\n"
     "     --hdd-bytes B  write B bytes per hdd worker (default is 1GB)\n"
     "     --hdd-noclean  do not unlink files created by hdd workers\n\n"
